﻿using System;
using System.Collections;
using System.Data;
using System.Data.Sql;
using System.Data.SqlClient;
using System.Drawing;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using Excel = Microsoft.Office.Interop.Excel;

namespace ExcelComputeBy
{
    class ExcelUILanguageHelper : IDisposable
    {
        private CultureInfo m_CurrentCulture;

        public ExcelUILanguageHelper()
        {
            // save current culture and set culture to en-US
            m_CurrentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
        }

        // return to normal culture
        public void Dispose() => Thread.CurrentThread.CurrentCulture = m_CurrentCulture;
    }

    class Program
    {
        private const string sqlselect = @"
            select 
                SalesOrderID, Quantity, UnitPrice, OrderTotal, [Name]
            from
            (
                SELECT 
                    SalesOrderID, 
                    productid as productid,
                    OrderQty as Quantity,
                    UnitPrice as UnitPrice,
                    OrderQty*UnitPrice as OrderTotal
                FROM Sales.SalesOrderDetail
                WHERE Year(ModifiedDate) = 2014 and MONTH(ModifiedDate) <= 2 and OrderQty > 1
            ) sales
            join
            (
                SELECT 
                    productid,
                    name
                FROM [Production].[Product]
            ) product
            on sales.productid = product.ProductID
            order by SalesOrderID, name
            ";
        private const string connectionString = "Server=TCP:192.168.0.7;Database=AdventureWorks2019;Integrated Security=SSPI;Connect Timeout=30;";

        static void Main(string[] args)
        {
            using(new ExcelUILanguageHelper())
            {
                string fileName = Environment.GetFolderPath(Environment.SpecialFolder.Desktop) + "\\" + "ExcelComputeBy.xlsx";

                Excel.Application xlsApp;
                Excel.Workbook xlsWorkbook;
                Excel.Worksheet xlsWorksheet;
                object misValue = System.Reflection.Missing.Value;

                // Remove the old excel report file
                try
                {
                    FileInfo oldFile = new FileInfo(fileName);
                    if(oldFile.Exists)
                    {
                        File.SetAttributes(oldFile.FullName, FileAttributes.Normal);
                        oldFile.Delete();
                    }
                }
                catch(Exception ex)
                {
                    MessageBox.Show("Error removing old Excel report: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }

                try
                {
                    xlsApp = new Excel.Application();
                    xlsWorkbook = xlsApp.Workbooks.Add(misValue);
                    xlsWorksheet = (Excel.Worksheet)xlsWorkbook.Sheets[1];

                    // Create the header for Excel file
                    xlsWorksheet.Cells[1, 1] = "Simulate COMPUTE BY clause when exporting data to Excel report";
                    Excel.Range range = xlsWorksheet.get_Range("A1", "E1");
                    range.Merge(1);
                    range.Borders.Color = Color.Black.ToArgb();
                    range.Interior.Color = Color.Yellow.ToArgb();

                    int i = 3;
                    int prevSalesOrderId = 0;
                    int currentSalesOrderId = 0;
                    double orderTotal = 0;

                    using SqlConnection conn = new SqlConnection(connectionString);
                    conn.Open();
                    using SqlCommand cmd = new SqlCommand(sqlselect, conn);
                    using SqlDataReader dr = cmd.ExecuteReader();
                    if(dr.HasRows)
                    {
                        for(int j = 0; j < dr.FieldCount; ++j)
                        {
                            xlsWorksheet.Cells[i, j + 1] = dr.GetName(j);
                        }
                        ++i;
                        // First read
                        dr.Read();
                        prevSalesOrderId = dr.GetInt32(0);
                        currentSalesOrderId = prevSalesOrderId;
                        orderTotal = (double)dr.GetDecimal(3);
                        for(int j = 1; j <= dr.FieldCount; ++j)
                            xlsWorksheet.Cells[i, j] = dr.GetValue(j - 1);
                        ++i;
                    }

                    while(dr.Read())
                    {
                        currentSalesOrderId = dr.GetInt32(0);
                        if(currentSalesOrderId == prevSalesOrderId)
                        {
                            orderTotal += (double)dr.GetDecimal(3);
                        }
                        else
                        {
                            // We have the first sum
                            xlsWorksheet.Cells[i, 4] = orderTotal;
                            orderTotal = (double)dr.GetDecimal(3);
                            prevSalesOrderId = currentSalesOrderId;
                            ++i;
                        }
                        for(int j = 1; j <= dr.FieldCount; ++j)
                            xlsWorksheet.Cells[i, j] = dr.GetValue(j - 1);
                        ++i;
                    }

                    range = xlsWorksheet.get_Range("A2", "I" + (i + 2).ToString());
                    range.Columns.AutoFit();

                    xlsWorkbook.SaveAs(fileName, Excel.XlFileFormat.xlWorkbookDefault, misValue, misValue, misValue, misValue,
                        Excel.XlSaveAsAccessMode.xlExclusive, Excel.XlSaveConflictResolution.xlLocalSessionChanges, misValue, misValue, misValue, misValue);
                    xlsWorkbook.Close(true, misValue, misValue);
                    xlsApp.Quit();

                    ReleaseObject(xlsWorksheet);
                    ReleaseObject(xlsWorkbook);
                    ReleaseObject(xlsApp);

                    if(MessageBox.Show("Excel report has been created on your desktop\nWould you like to open it?", "Created Excel report",
                        MessageBoxButtons.YesNo, MessageBoxIcon.Information, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                    {
                        Process.Start(fileName);
                    }
                }
                catch(Exception ex)
                {
                    MessageBox.Show("Error creating Excel report: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }
            }
        }

        static private void ReleaseObject(object obj)
        {
            try
            {
                System.Runtime.InteropServices.Marshal.ReleaseComObject(obj);
                obj = null;
            }
            catch(Exception ex)
            {
                obj = null;
                MessageBox.Show("Exception occured while releasing object " + ex.ToString());
            }
            finally
            {
                GC.Collect();
            }
        }
    }
}
